"""Tests for data models."""

import pytest
from datetime import datetime

from wifi_profiler.core.models import (
    ConnectionQuality,
    InterfaceInfo,
    WiFiStats,
    PingResult,
    PingSummary,
    ThroughputResult,
    TestSession,
    Config
)


class TestConnectionQuality:
    """Test ConnectionQuality enum."""
    
    def test_enum_values(self):
        """Test enum values are correct."""
        assert ConnectionQuality.EXCELLENT == "excellent"
        assert ConnectionQuality.GOOD == "good"
        assert ConnectionQuality.FAIR == "fair"
        assert ConnectionQuality.POOR == "poor"
        assert ConnectionQuality.UNKNOWN == "unknown"


class TestInterfaceInfo:
    """Test InterfaceInfo model."""
    
    def test_create_interface_info(self):
        """Test creating InterfaceInfo instance."""
        info = InterfaceInfo(
            name="wlan0",
            driver="iwlwifi",
            ssid="MyWiFi",
            frequency=2.4,
            channel=6,
            bitrate=866.7,
            is_connected=True
        )
        
        assert info.name == "wlan0"
        assert info.driver == "iwlwifi"
        assert info.ssid == "MyWiFi"
        assert info.frequency == 2.4
        assert info.channel == 6
        assert info.bitrate == 866.7
        assert info.is_connected is True
        assert info.is_wireless is True
    
    def test_minimal_interface_info(self):
        """Test creating InterfaceInfo with minimal data."""
        info = InterfaceInfo(name="wlan0")
        
        assert info.name == "wlan0"
        assert info.driver is None
        assert info.ssid is None
        assert info.is_connected is False


class TestWiFiStats:
    """Test WiFiStats model."""
    
    def test_create_wifi_stats(self):
        """Test creating WiFiStats instance."""
        stats = WiFiStats(
            rssi=-45.2,
            noise=-85.0,
            signal_quality=85.5,
            tx_retries=5,
            rx_drop=2,
            tx_drop=1
        )
        
        assert stats.rssi == -45.2
        assert stats.noise == -85.0
        assert stats.signal_quality == 85.5
        assert stats.tx_retries == 5
        assert stats.rx_drop == 2
        assert stats.tx_drop == 1
        assert isinstance(stats.timestamp, datetime)
    
    def test_empty_wifi_stats(self):
        """Test creating empty WiFiStats."""
        stats = WiFiStats()
        
        assert stats.rssi is None
        assert stats.noise is None
        assert stats.signal_quality is None
        assert isinstance(stats.timestamp, datetime)


class TestPingResult:
    """Test PingResult model."""
    
    def test_create_ping_result(self):
        """Test creating PingResult instance."""
        result = PingResult(
            target="8.8.8.8",
            sequence=1,
            response_time=12.3,
            ttl=64,
            size=64,
            success=True
        )
        
        assert result.target == "8.8.8.8"
        assert result.sequence == 1
        assert result.response_time == 12.3
        assert result.ttl == 64
        assert result.size == 64
        assert result.success is True
        assert result.error is None
    
    def test_failed_ping_result(self):
        """Test creating failed PingResult."""
        result = PingResult(
            target="invalid.host",
            sequence=1,
            success=False,
            error="Name or service not known"
        )
        
        assert result.target == "invalid.host"
        assert result.success is False
        assert result.error == "Name or service not known"
        assert result.response_time is None
    
    def test_negative_response_time_validation(self):
        """Test that negative response times are rejected."""
        with pytest.raises(ValueError):
            PingResult(
                target="8.8.8.8",
                sequence=1,
                response_time=-1.0,
                success=True
            )


class TestPingSummary:
    """Test PingSummary model."""
    
    def test_create_ping_summary(self):
        """Test creating PingSummary instance."""
        summary = PingSummary(
            target="8.8.8.8",
            total_sent=10,
            total_received=9,
            packet_loss_percent=10.0,
            min_rtt=10.1,
            avg_rtt=12.5,
            max_rtt=15.2,
            mdev_rtt=1.2
        )
        
        assert summary.target == "8.8.8.8"
        assert summary.total_sent == 10
        assert summary.total_received == 9
        assert summary.packet_loss_percent == 10.0
        assert summary.min_rtt == 10.1
        assert summary.avg_rtt == 12.5
        assert summary.max_rtt == 15.2
        assert summary.mdev_rtt == 1.2
    
    def test_success_rate_calculation(self):
        """Test success rate calculation."""
        summary = PingSummary(
            target="8.8.8.8",
            total_sent=10,
            total_received=8,
            packet_loss_percent=20.0
        )
        
        assert summary.success_rate == 80.0
    
    def test_success_rate_zero_sent(self):
        """Test success rate when no packets sent."""
        summary = PingSummary(
            target="8.8.8.8",
            total_sent=0,
            total_received=0,
            packet_loss_percent=100.0
        )
        
        assert summary.success_rate == 0.0
    
    def test_quality_assessment(self):
        """Test quality assessment based on metrics."""
        # Excellent quality
        excellent = PingSummary(
            target="8.8.8.8",
            total_sent=10,
            total_received=10,
            packet_loss_percent=0.0,
            avg_rtt=25.0
        )
        assert excellent.quality == ConnectionQuality.EXCELLENT
        
        # Poor quality
        poor = PingSummary(
            target="8.8.8.8",
            total_sent=10,
            total_received=8,
            packet_loss_percent=15.0
        )
        assert poor.quality == ConnectionQuality.POOR


class TestThroughputResult:
    """Test ThroughputResult model."""
    
    def test_create_throughput_result(self):
        """Test creating ThroughputResult instance."""
        result = ThroughputResult(
            direction="download",
            speed_mbps=100.5,
            duration_seconds=10.0,
            bytes_transferred=125625000,
            test_type="iperf3",
            target="speedtest.tele2.net:5201"
        )
        
        assert result.direction == "download"
        assert result.speed_mbps == 100.5
        assert result.duration_seconds == 10.0
        assert result.bytes_transferred == 125625000
        assert result.test_type == "iperf3"
        assert result.target == "speedtest.tele2.net:5201"
    
    def test_speed_formatting(self):
        """Test speed formatting."""
        # Mbps
        result_mbps = ThroughputResult(
            direction="download",
            speed_mbps=100.5,
            duration_seconds=10.0,
            bytes_transferred=125625000,
            test_type="iperf3"
        )
        assert result_mbps.speed_mbps_formatted == "100.50 Mbps"
        
        # Gbps
        result_gbps = ThroughputResult(
            direction="download",
            speed_mbps=1500.0,
            duration_seconds=10.0,
            bytes_transferred=1875000000,
            test_type="iperf3"
        )
        assert result_gbps.speed_mbps_formatted == "1.50 Gbps"


class TestTestSession:
    """Test TestSession model."""
    
    def test_create_test_session(self):
        """Test creating TestSession instance."""
        start_time = datetime.now()
        session = TestSession(
            session_id="test_123",
            start_time=start_time,
            config={"interface": "wlan0", "duration": 30}
        )
        
        assert session.session_id == "test_123"
        assert session.start_time == start_time
        assert session.end_time is None
        assert session.interface is None
        assert session.wifi_stats == []
        assert session.ping_results == []
        assert session.ping_summaries == []
        assert session.throughput_results == []
        assert session.config == {"interface": "wlan0", "duration": 30}
    
    def test_session_duration(self):
        """Test session duration calculation."""
        start_time = datetime.now()
        session = TestSession(
            session_id="test_123",
            start_time=start_time
        )
        
        # No end time
        assert session.duration is None
        
        # With end time
        session.end_time = start_time
        assert session.duration == 0.0
    
    def test_session_completion(self):
        """Test session completion status."""
        session = TestSession(
            session_id="test_123",
            start_time=datetime.now()
        )
        
        assert session.is_complete is False
        
        session.end_time = datetime.now()
        assert session.is_complete is True


class TestConfig:
    """Test Config model."""
    
    def test_default_config(self):
        """Test default configuration values."""
        config = Config()
        
        assert config.default_interface is None
        assert config.default_targets == ["8.8.8.8", "1.1.1.1"]
        assert config.default_duration == 30
        assert config.ping_count == 10
        assert config.ping_interval == 1.0
        assert config.throughput_test_duration == 10
        assert config.iperf3_server is None
        assert config.http_download_url == "https://speed.hetzner.de/100MB.bin"
        assert config.log_level == "INFO"
        assert config.output_format == "text"
        assert config.save_sessions is True
        assert config.session_dir == "~/.local/share/wifi-profiler/sessions"
    
    def test_custom_config(self):
        """Test custom configuration values."""
        config = Config(
            default_interface="wlan0",
            default_targets=["192.168.1.1"],
            default_duration=60,
            log_level="DEBUG"
        )
        
        assert config.default_interface == "wlan0"
        assert config.default_targets == ["192.168.1.1"]
        assert config.default_duration == 60
        assert config.log_level == "DEBUG" 