"""Main CLI interface for WiFi profiler."""

import asyncio
import json
import sys
from pathlib import Path
from typing import List, Optional
from datetime import datetime

import typer
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.text import Text

from .core.models import (
    Config, 
    TestSession, 
    SessionSummary, 
    ConnectionQuality,
    InterfaceInfo
)
from .core.system_detect import get_system_info, check_network_connectivity
from .core.wifi_stats import WiFiStatsCollector, list_wireless_interfaces
from .core.ping import PingTester, ping_hosts, format_ping_summary
from .core.throughput import ThroughputTester, test_bandwidth, format_throughput_result
from .tui.live_view import LiveViewApp
from .utils.shell import ShellError

app = typer.Typer(
    name="wifi-profiler",
    help="A cross-platform CLI/TUI tool for measuring and reporting Wi-Fi quality",
    add_completion=False,
    rich_markup_mode="rich",
    no_args_is_help=True
)

console = Console()


def load_config() -> Config:
    """Load configuration from file or use defaults."""
    config_path = Path.home() / ".config" / "wifi-profiler" / "config.toml"
    
    if config_path.exists():
        try:
            import tomli
            with open(config_path, "rb") as f:
                config_data = tomli.load(f)
            return Config(**config_data)
        except Exception as e:
            console.print(f"[yellow]Warning: Could not load config from {config_path}: {e}[/yellow]")
    
    return Config()


def save_session(session: TestSession, output_path: Optional[Path] = None) -> None:
    """Save session data to file."""
    if output_path is None:
        output_path = Path(f"wifi_profiler_session_{session.session_id}.json")
    
    try:
        with open(output_path, "w") as f:
            json.dump(session.model_dump(), f, indent=2, default=str)
        console.print(f"[green]Session saved to {output_path}[/green]")
    except Exception as e:
        console.print(f"[red]Error saving session: {e}[/red]")


def print_session_summary(session: TestSession) -> None:
    """Print a summary of the test session."""
    console.print(Panel(
        f"[bold]Session Summary[/bold]\n"
        f"Session ID: {session.session_id}\n"
        f"Duration: {session.duration:.1f}s\n"
        f"Interface: {session.interface.name if session.interface else 'Unknown'}\n"
        f"SSID: {session.interface.ssid if session.interface else 'Unknown'}\n"
        f"Tests: {len(session.ping_summaries)} ping, {len(session.throughput_results)} throughput",
        title="Test Results",
        border_style="green"
    ))


@app.command()
def quick(
    iface: Optional[str] = typer.Option(None, "--iface", "-i", help="Wireless interface name"),
    target: List[str] = typer.Option(["8.8.8.8"], "--target", "-t", help="Ping targets"),
    duration: int = typer.Option(10, "--duration", "-d", help="Test duration in seconds"),
    json_output: Optional[Path] = typer.Option(None, "--json", help="Save results to JSON file"),
    html_output: Optional[Path] = typer.Option(None, "--html", help="Generate HTML report"),
    export: Optional[Path] = typer.Option(None, "--export", help="Save session data"),
    verbose: bool = typer.Option(False, "--verbose", "-v", help="Verbose output"),
    quiet: bool = typer.Option(False, "--quiet", "-q", help="Quiet mode")
):
    """Quick WiFi quality test (10s ping + RSSI)."""
    
    if quiet:
        console.quiet = True
    
    # Check system requirements
    system_info = get_system_info()
    if verbose:
        system_info.print_system_info()
    
    requirements_met, issues = system_info.check_requirements()
    if not requirements_met:
        console.print("[red]System requirements not met:[/red]")
        for issue in issues:
            console.print(f"  - {issue}")
        raise typer.Exit(1)
    
    # Check network connectivity
    if not check_network_connectivity():
        console.print("[red]No network connectivity detected[/red]")
        raise typer.Exit(1)
    
    async def run_quick_test():
        # Get interface info
        if iface:
            interface_name = iface
        else:
            interfaces = await list_wireless_interfaces()
            if not interfaces:
                console.print("[red]No wireless interfaces found[/red]")
                return
            interface_name = interfaces[0].name
            if verbose:
                console.print(f"Using interface: {interface_name}")
        
        # Create session
        session = TestSession(
            session_id=f"quick_{int(datetime.now().timestamp())}",
            start_time=datetime.now(),
            config={
                "interface": interface_name,
                "targets": target,
                "duration": duration
            }
        )
        
        # Get interface details
        collector = WiFiStatsCollector()
        session.interface = await collector.get_interface_info(interface_name)
        
        if session.interface:
            console.print(f"Interface: {session.interface.name}")
            if session.interface.ssid:
                console.print(f"SSID: {session.interface.ssid}")
            if session.interface.bitrate:
                console.print(f"Bitrate: {session.interface.bitrate} Mbps")
        
        # Get WiFi stats
        wifi_stats = await collector.get_wifi_stats(interface_name)
        if wifi_stats:
            session.wifi_stats.append(wifi_stats)
            if wifi_stats.rssi:
                console.print(f"RSSI: {wifi_stats.rssi} dBm")
            if wifi_stats.signal_quality:
                console.print(f"Signal Quality: {wifi_stats.signal_quality:.1f}%")
        
        # Run ping tests
        console.print(f"\nPinging {len(target)} targets...")
        ping_summaries = await ping_hosts(target, count=4)
        session.ping_summaries = ping_summaries
        
        for summary in ping_summaries:
            console.print(format_ping_summary(summary))
        
        # End session
        session.end_time = datetime.now()
        
        # Print summary
        print_session_summary(session)
        
        # Save outputs
        if json_output:
            save_session(session, json_output)
        
        if html_output:
            from .report_html import generate_html_report
            generate_html_report(session, html_output)
            console.print(f"[green]HTML report saved to {html_output}[/green]")
        
        if export:
            save_session(session, export)
    
    asyncio.run(run_quick_test())


@app.command()
def live(
    iface: Optional[str] = typer.Option(None, "--iface", "-i", help="Wireless interface name"),
    target: List[str] = typer.Option(["8.8.8.8"], "--target", "-t", help="Ping targets"),
    duration: int = typer.Option(300, "--duration", "-d", help="Test duration in seconds"),
    interval: float = typer.Option(1.0, "--interval", help="Update interval in seconds")
):
    """Live TUI dashboard with real-time WiFi metrics."""
    
    # Check system requirements
    system_info = get_system_info()
    requirements_met, issues = system_info.check_requirements()
    if not requirements_met:
        console.print("[red]System requirements not met:[/red]")
        for issue in issues:
            console.print(f"  - {issue}")
        raise typer.Exit(1)
    
    # Get interface
    if iface:
        interface_name = iface
    else:
        interfaces = asyncio.run(list_wireless_interfaces())
        if not interfaces:
            console.print("[red]No wireless interfaces found[/red]")
            raise typer.Exit(1)
        interface_name = interfaces[0].name
        console.print(f"Using interface: {interface_name}")
    
    # Start TUI
    app = LiveViewApp(
        interface=interface_name,
        targets=target,
        duration=duration,
        interval=interval
    )
    app.run()


@app.command()
def survey(
    iface: Optional[str] = typer.Option(None, "--iface", "-i", help="Wireless interface name"),
    target: List[str] = typer.Option(["8.8.8.8", "1.1.1.1"], "--target", "-t", help="Ping targets"),
    duration: int = typer.Option(30, "--duration", "-d", help="Test duration in seconds"),
    ping_count: int = typer.Option(10, "--ping-count", help="Number of ping packets"),
    throughput_duration: int = typer.Option(10, "--throughput-duration", help="Throughput test duration"),
    json_output: Optional[Path] = typer.Option(None, "--json", help="Save results to JSON file"),
    html_output: Optional[Path] = typer.Option(None, "--html", help="Generate HTML report"),
    export: Optional[Path] = typer.Option(None, "--export", help="Save session data"),
    verbose: bool = typer.Option(False, "--verbose", "-v", help="Verbose output")
):
    """Comprehensive WiFi survey with ping, throughput, and detailed analysis."""
    
    # Check system requirements
    system_info = get_system_info()
    if verbose:
        system_info.print_system_info()
    
    requirements_met, issues = system_info.check_requirements()
    if not requirements_met:
        console.print("[red]System requirements not met:[/red]")
        for issue in issues:
            console.print(f"  - {issue}")
        raise typer.Exit(1)
    
    async def run_survey():
        # Get interface info
        if iface:
            interface_name = iface
        else:
            interfaces = await list_wireless_interfaces()
            if not interfaces:
                console.print("[red]No wireless interfaces found[/red]")
                return
            interface_name = interfaces[0].name
            if verbose:
                console.print(f"Using interface: {interface_name}")
        
        # Create session
        session = TestSession(
            session_id=f"survey_{int(datetime.now().timestamp())}",
            start_time=datetime.now(),
            config={
                "interface": interface_name,
                "targets": target,
                "duration": duration,
                "ping_count": ping_count,
                "throughput_duration": throughput_duration
            }
        )
        
        # Get interface details
        collector = WiFiStatsCollector()
        session.interface = await collector.get_interface_info(interface_name)
        
        if session.interface:
            console.print(f"Interface: {session.interface.name}")
            if session.interface.ssid:
                console.print(f"SSID: {session.interface.ssid}")
            if session.interface.bitrate:
                console.print(f"Bitrate: {session.interface.bitrate} Mbps")
        
        # Monitor WiFi stats during the test
        console.print(f"\nMonitoring WiFi stats for {duration} seconds...")
        wifi_stats_list = await collector.monitor_wifi_stats(
            interface_name, 
            duration=duration, 
            interval=1.0
        )
        session.wifi_stats = wifi_stats_list
        
        # Calculate average RSSI
        rssi_values = [stats.rssi for stats in wifi_stats_list if stats.rssi is not None]
        if rssi_values:
            avg_rssi = sum(rssi_values) / len(rssi_values)
            console.print(f"Average RSSI: {avg_rssi:.1f} dBm")
        
        # Run ping tests
        console.print(f"\nRunning ping tests to {len(target)} targets...")
        ping_tester = PingTester()
        ping_summaries = await ping_tester.ping_multiple_targets(target, ping_count)
        session.ping_summaries = ping_summaries
        
        for summary in ping_summaries:
            console.print(format_ping_summary(summary))
        
        # Run throughput tests
        console.print(f"\nRunning throughput tests...")
        throughput_tester = ThroughputTester()
        throughput_results = await throughput_tester.test_multiple_directions(
            duration=throughput_duration
        )
        session.throughput_results = throughput_results
        
        for result in throughput_results:
            console.print(format_throughput_result(result))
        
        # End session
        session.end_time = datetime.now()
        
        # Print comprehensive summary
        print_session_summary(session)
        
        # Calculate overall quality
        if session.ping_summaries:
            avg_packet_loss = sum(s.packet_loss_percent for s in session.ping_summaries) / len(session.ping_summaries)
            avg_latency = sum(s.avg_rtt for s in session.ping_summaries if s.avg_rtt) / len([s for s in session.ping_summaries if s.avg_rtt])
            
            if avg_packet_loss > 10:
                overall_quality = ConnectionQuality.POOR
            elif avg_packet_loss > 5 or avg_latency > 100:
                overall_quality = ConnectionQuality.FAIR
            elif avg_packet_loss < 1 and avg_latency < 50:
                overall_quality = ConnectionQuality.EXCELLENT
            else:
                overall_quality = ConnectionQuality.GOOD
            
            console.print(f"Overall Quality: {overall_quality.value}")
        
        # Save outputs
        if json_output:
            save_session(session, json_output)
        
        if html_output:
            from .report_html import generate_html_report
            generate_html_report(session, html_output)
            console.print(f"[green]HTML report saved to {html_output}[/green]")
        
        if export:
            save_session(session, export)
    
    asyncio.run(run_survey())


@app.command()
def devices(
    verbose: bool = typer.Option(False, "--verbose", "-v", help="Verbose output"),
    json_output: bool = typer.Option(False, "--json", help="Output in JSON format")
):
    """List wireless interfaces and their details."""
    
    async def list_devices():
        interfaces = await list_wireless_interfaces()
        
        if json_output:
            json.dump([iface.model_dump() for iface in interfaces], sys.stdout, indent=2, default=str)
            return
        
        if not interfaces:
            console.print("[yellow]No wireless interfaces found[/yellow]")
            return
        
        # Create table
        table = Table(title="Wireless Interfaces")
        table.add_column("Interface", style="cyan")
        table.add_column("Driver", style="magenta")
        table.add_column("SSID", style="green")
        table.add_column("Connected", style="yellow")
        table.add_column("Bitrate", style="blue")
        table.add_column("Frequency", style="red")
        
        for iface in interfaces:
            table.add_row(
                iface.name,
                iface.driver or "Unknown",
                iface.ssid or "Not connected",
                "✓" if iface.is_connected else "✗",
                f"{iface.bitrate:.0f} Mbps" if iface.bitrate else "Unknown",
                f"{iface.frequency:.1f} GHz" if iface.frequency else "Unknown"
            )
        
        console.print(table)
        
        if verbose:
            # Show system info
            system_info = get_system_info()
            system_info.print_system_info()
    
    asyncio.run(list_devices())


@app.command()
def import_session(
    session_file: Path = typer.Argument(..., help="Session file to import"),
    html_output: Optional[Path] = typer.Option(None, "--html", help="Generate HTML report"),
    json_output: Optional[Path] = typer.Option(None, "--json", help="Save as JSON")
):
    """Import a saved session and generate reports."""
    
    if not session_file.exists():
        console.print(f"[red]Session file not found: {session_file}[/red]")
        raise typer.Exit(1)
    
    try:
        with open(session_file, "r") as f:
            session_data = json.load(f)
        
        session = TestSession(**session_data)
        
        console.print(f"Imported session: {session.session_id}")
        print_session_summary(session)
        
        if html_output:
            from .report_html import generate_html_report
            generate_html_report(session, html_output)
            console.print(f"[green]HTML report saved to {html_output}[/green]")
        
        if json_output:
            save_session(session, json_output)
    
    except Exception as e:
        console.print(f"[red]Error importing session: {e}[/red]")
        raise typer.Exit(1)


@app.command()
def serve(
    port: int = typer.Option(8080, "--port", "-p", help="Port to serve on"),
    host: str = typer.Option("localhost", "--host", "-h", help="Host to bind to"),
    interface: Optional[str] = typer.Option(None, "--iface", "-i", help="Wireless interface to monitor")
):
    """Start a simple HTTP server to serve metrics."""
    
    console.print(f"Starting metrics server on {host}:{port}")
    console.print("This feature is not yet implemented.")
    console.print("Use 'wifi-profiler live' for real-time monitoring instead.")
    raise typer.Exit(1)


@app.command()
def version():
    """Show version and exit."""
    from . import __version__
    console.print(f"WiFi Profiler v{__version__}")


@app.command()
def config():
    """Show configuration and exit."""
    config_obj = load_config()
    console.print(Panel(
        f"[bold]Configuration[/bold]\n"
        f"Default Interface: {config_obj.default_interface or 'Auto'}\n"
        f"Default Targets: {', '.join(config_obj.default_targets)}\n"
        f"Default Duration: {config_obj.default_duration}s\n"
        f"Ping Count: {config_obj.ping_count}\n"
        f"Throughput Duration: {config_obj.throughput_test_duration}s\n"
        f"HTTP Download URL: {config_obj.http_download_url}\n"
        f"Log Level: {config_obj.log_level}",
        title="WiFi Profiler Config",
        border_style="blue"
    ))


if __name__ == "__main__":
    app() 