"""Data models for WiFi profiler results and configurations."""

from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional, Union
from pydantic import BaseModel, Field, validator


class ConnectionQuality(str, Enum):
    """WiFi connection quality levels."""
    EXCELLENT = "excellent"
    GOOD = "good"
    FAIR = "fair"
    POOR = "poor"
    UNKNOWN = "unknown"


class InterfaceInfo(BaseModel):
    """Information about a wireless interface."""
    name: str
    driver: Optional[str] = None
    ssid: Optional[str] = None
    bssid: Optional[str] = None
    frequency: Optional[float] = None  # GHz
    channel: Optional[int] = None
    tx_power: Optional[float] = None  # dBm
    bitrate: Optional[float] = None  # Mbps
    is_connected: bool = False
    is_wireless: bool = True


class WiFiStats(BaseModel):
    """Current WiFi statistics."""
    rssi: Optional[float] = Field(None, description="Signal strength in dBm")
    noise: Optional[float] = Field(None, description="Noise level in dBm")
    signal_quality: Optional[float] = Field(None, description="Signal quality percentage")
    tx_retries: Optional[int] = Field(None, description="Transmission retries")
    rx_drop: Optional[int] = Field(None, description="Received packets dropped")
    tx_drop: Optional[int] = Field(None, description="Transmitted packets dropped")
    timestamp: datetime = Field(default_factory=datetime.now)


class PingResult(BaseModel):
    """Result of a ping test."""
    target: str
    sequence: int
    response_time: Optional[float] = Field(None, description="Response time in ms")
    ttl: Optional[int] = Field(None, description="Time to live")
    size: Optional[int] = Field(None, description="Packet size in bytes")
    timestamp: datetime = Field(default_factory=datetime.now)
    success: bool = True
    error: Optional[str] = None

    @validator('response_time')
    def validate_response_time(cls, v):
        if v is not None and v < 0:
            raise ValueError('Response time cannot be negative')
        return v


class PingSummary(BaseModel):
    """Summary statistics for ping tests."""
    target: str
    total_sent: int
    total_received: int
    packet_loss_percent: float
    min_rtt: Optional[float] = None
    avg_rtt: Optional[float] = None
    max_rtt: Optional[float] = None
    mdev_rtt: Optional[float] = None  # Mean deviation (jitter)
    timestamp: datetime = Field(default_factory=datetime.now)

    @property
    def success_rate(self) -> float:
        """Calculate success rate as percentage."""
        if self.total_sent == 0:
            return 0.0
        return (self.total_received / self.total_sent) * 100

    @property
    def quality(self) -> ConnectionQuality:
        """Determine connection quality based on packet loss and latency."""
        if self.packet_loss_percent > 10:
            return ConnectionQuality.POOR
        elif self.packet_loss_percent > 5:
            return ConnectionQuality.FAIR
        elif self.avg_rtt and self.avg_rtt > 100:
            return ConnectionQuality.FAIR
        elif self.packet_loss_percent < 1 and (not self.avg_rtt or self.avg_rtt < 50):
            return ConnectionQuality.EXCELLENT
        else:
            return ConnectionQuality.GOOD


class ThroughputResult(BaseModel):
    """Result of a throughput test."""
    direction: str = Field(..., description="upload or download")
    speed_mbps: float
    duration_seconds: float
    bytes_transferred: int
    timestamp: datetime = Field(default_factory=datetime.now)
    test_type: str = Field(..., description="iperf3, http_download, or custom")
    target: Optional[str] = None
    error: Optional[str] = None

    @property
    def speed_mbps_formatted(self) -> str:
        """Format speed with appropriate units."""
        if self.speed_mbps >= 1000:
            return f"{self.speed_mbps/1000:.2f} Gbps"
        return f"{self.speed_mbps:.2f} Mbps"


class TestSession(BaseModel):
    """A complete test session with all results."""
    session_id: str
    start_time: datetime
    end_time: Optional[datetime] = None
    interface: Optional[InterfaceInfo] = None
    wifi_stats: List[WiFiStats] = []
    ping_results: List[PingResult] = []
    ping_summaries: List[PingSummary] = []
    throughput_results: List[ThroughputResult] = []
    config: Dict[str, Union[str, int, float, bool, List[str]]] = Field(default_factory=dict)

    @property
    def duration(self) -> Optional[float]:
        """Calculate session duration in seconds."""
        if self.end_time:
            return (self.end_time - self.start_time).total_seconds()
        return None

    @property
    def is_complete(self) -> bool:
        """Check if the session is complete."""
        return self.end_time is not None


class SessionSummary(BaseModel):
    """Summary of a test session."""
    session_id: str
    start_time: datetime
    end_time: Optional[datetime] = None
    interface_name: Optional[str] = None
    ssid: Optional[str] = None
    overall_quality: ConnectionQuality = ConnectionQuality.UNKNOWN
    avg_rssi: Optional[float] = None
    avg_latency: Optional[float] = None
    packet_loss_percent: Optional[float] = None
    max_throughput: Optional[float] = None
    test_count: int = 0
    error_count: int = 0

    @property
    def duration(self) -> Optional[float]:
        """Calculate session duration in seconds."""
        if self.end_time:
            return (self.end_time - self.start_time).total_seconds()
        return None


class Config(BaseModel):
    """Configuration for WiFi profiler."""
    default_interface: Optional[str] = None
    default_targets: List[str] = Field(default_factory=lambda: ["8.8.8.8", "1.1.1.1"])
    default_duration: int = 30
    ping_count: int = 10
    ping_interval: float = 1.0
    throughput_test_duration: int = 10
    iperf3_server: Optional[str] = None
    http_download_url: str = "https://speed.hetzner.de/100MB.bin"
    log_level: str = "INFO"
    output_format: str = "text"  # text, json, html
    save_sessions: bool = True
    session_dir: str = "~/.local/share/wifi-profiler/sessions" 