"""Ping testing for latency, jitter, and packet loss measurements."""

import asyncio
import re
import statistics
from typing import List, Optional, Tuple
from datetime import datetime

from .models import PingResult, PingSummary, ConnectionQuality
from .system_detect import get_system_info
from ..utils.shell import run_command, ShellError

from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn

console = Console()


class PingTester:
    """Perform ping tests and calculate statistics."""
    
    def __init__(self):
        self.system_info = get_system_info()
    
    async def ping_target(
        self, 
        target: str, 
        count: int = 4, 
        interval: float = 1.0,
        timeout: float = 5.0
    ) -> List[PingResult]:
        """
        Ping a target and return detailed results.
        
        Args:
            target: Target hostname or IP address
            count: Number of ping packets to send
            interval: Interval between pings in seconds
            timeout: Timeout for each ping in seconds
        
        Returns:
            List of PingResult objects
        """
        results = []
        
        # Get the appropriate ping command for the system
        cmd = self.system_info.get_ping_command(target, count, interval)
        
        try:
            # Run the ping command
            returncode, stdout, stderr = await run_command(
                cmd, 
                timeout=timeout + (count * interval),
                show_progress=True,
                progress_text=f"Pinging {target}..."
            )
            

            
            # Parse the output
            results = self._parse_ping_output(stdout, target)
            
            # If we didn't get enough results, create failed results
            while len(results) < count:
                results.append(PingResult(
                    target=target,
                    sequence=len(results) + 1,
                    success=False,
                    error="Timeout or no response"
                ))
            
        except ShellError as e:
            console.print(f"[red]Ping failed for {target}: {e}[/red]")
            # Create failed results
            for i in range(count):
                results.append(PingResult(
                    target=target,
                    sequence=i + 1,
                    success=False,
                    error=str(e)
                ))
        
        except asyncio.TimeoutError:
            console.print(f"[red]Ping timeout for {target}[/red]")
            # Create failed results
            for i in range(count):
                results.append(PingResult(
                    target=target,
                    sequence=i + 1,
                    success=False,
                    error="Timeout"
                ))
        
        return results
    
    async def _ping_target_no_progress(
        self, 
        target: str, 
        count: int = 4, 
        interval: float = 1.0,
        timeout: float = 5.0
    ) -> List[PingResult]:
        """Ping a target without progress display (for concurrent operations)."""
        results = []
        
        # Get the appropriate ping command for the system
        cmd = self.system_info.get_ping_command(target, count, interval)
        
        try:
            # Run the ping command without progress
            returncode, stdout, stderr = await run_command(
                cmd, 
                timeout=timeout + (count * interval),
                show_progress=False
            )
            
            # Parse the output
            results = self._parse_ping_output(stdout, target)
            
            # If we didn't get enough results, create failed results
            while len(results) < count:
                results.append(PingResult(
                    target=target,
                    sequence=len(results) + 1,
                    success=False,
                    error="Timeout or no response"
                ))
            
        except ShellError as e:
            # Create failed results
            for i in range(count):
                results.append(PingResult(
                    target=target,
                    sequence=i + 1,
                    success=False,
                    error=str(e)
                ))
        
        except asyncio.TimeoutError:
            # Create failed results
            for i in range(count):
                results.append(PingResult(
                    target=target,
                    sequence=i + 1,
                    success=False,
                    error="Timeout"
                ))
        
        return results
    
    def _parse_ping_output(self, output: str, target: str) -> List[PingResult]:
        """Parse ping command output and extract results."""
        results = []
        
        if self.system_info.os_type == "linux":
            results = self._parse_linux_ping_output(output, target)
        elif self.system_info.os_type == "macos":
            results = self._parse_macos_ping_output(output, target)
        else:
            results = self._parse_generic_ping_output(output, target)
        
        return results
    
    def _parse_linux_ping_output(self, output: str, target: str) -> List[PingResult]:
        """Parse Linux ping output."""
        results = []
        
        # Linux ping output format:
        # 64 bytes from 8.8.8.8: icmp_seq=1 ttl=118 time=12.3 ms
        # 64 bytes from 8.8.8.8: icmp_seq=2 ttl=118 time=11.8 ms
        # ...
        # --- 8.8.8.8 ping statistics ---
        # 4 packets transmitted, 4 received, 0% packet loss, time 3003ms
        # rtt min/avg/max/mdev = 11.800/12.050/12.300/0.200 ms
        
        lines = output.split('\n')
        for line in lines:
            line = line.strip()
            
            # Match successful ping responses
            match = re.search(r'(\d+) bytes from .*: icmp_seq=(\d+) ttl=\d+ time=([\d.]+) ms', line)
            if match:
                size = int(match.group(1))
                sequence = int(match.group(2))
                response_time = float(match.group(3))
                
                results.append(PingResult(
                    target=target,
                    sequence=sequence,
                    response_time=response_time,
                    size=size,
                    success=True
                ))
            
            # Match failed pings (timeout)
            elif 'no answer yet' in line or 'Destination Host Unreachable' in line:
                # Try to extract sequence number
                seq_match = re.search(r'icmp_seq=(\d+)', line)
                if seq_match:
                    sequence = int(seq_match.group(1))
                    results.append(PingResult(
                        target=target,
                        sequence=sequence,
                        success=False,
                        error="No response"
                    ))
        
        return results
    
    def _parse_macos_ping_output(self, output: str, target: str) -> List[PingResult]:
        """Parse macOS ping output."""
        results = []
        
        # macOS ping output format:
        # 64 bytes from 8.8.8.8: icmp_seq=1 time=12.345 ms
        # 64 bytes from 8.8.8.8: icmp_seq=2 time=11.789 ms
        # ...
        # --- 8.8.8.8 ping statistics ---
        # 4 packets transmitted, 4 packets received, 0.0% packet loss
        # round-trip min/avg/max/stddev = 11.789/12.067/12.345/0.278 ms
        
        lines = output.split('\n')
        for line in lines:
            line = line.strip()
            
            # Match successful ping responses (macOS might not have ttl in output)
            match = re.search(r'(\d+) bytes from .*: icmp_seq=(\d+)(?: ttl=\d+)? time=([\d.]+) ms', line)
            if match:
                size = int(match.group(1))
                sequence = int(match.group(2))
                response_time = float(match.group(3))
                
                results.append(PingResult(
                    target=target,
                    sequence=sequence,
                    response_time=response_time,
                    size=size,
                    success=True
                ))
            
            # Match failed pings
            elif 'Request timeout' in line or 'No route to host' in line:
                # Try to extract sequence number
                seq_match = re.search(r'icmp_seq=(\d+)', line)
                if seq_match:
                    sequence = int(seq_match.group(1))
                    results.append(PingResult(
                        target=target,
                        sequence=sequence,
                        success=False,
                        error="Timeout"
                    ))
        
        return results
    
    def _parse_generic_ping_output(self, output: str, target: str) -> List[PingResult]:
        """Parse generic ping output (fallback)."""
        results = []
        
        # Try to match common ping response patterns
        lines = output.split('\n')
        for line in lines:
            line = line.strip()
            
            # Generic pattern for ping responses
            match = re.search(r'(\d+).*bytes.*time[=:]\s*([\d.]+)\s*ms', line, re.IGNORECASE)
            if match:
                size = int(match.group(1))
                response_time = float(match.group(2))
                
                # Try to extract sequence number
                seq_match = re.search(r'seq[=:]\s*(\d+)', line, re.IGNORECASE)
                sequence = int(seq_match.group(1)) if seq_match else len(results) + 1
                
                results.append(PingResult(
                    target=target,
                    sequence=sequence,
                    response_time=response_time,
                    size=size,
                    success=True
                ))
        
        return results
    
    def calculate_ping_summary(self, results: List[PingResult]) -> PingSummary:
        """Calculate summary statistics from ping results."""
        if not results:
            return PingSummary(
                target="unknown",
                total_sent=0,
                total_received=0,
                packet_loss_percent=100.0
            )
        
        target = results[0].target
        total_sent = len(results)
        successful_results = [r for r in results if r.success and r.response_time is not None]
        total_received = len(successful_results)
        
        # Calculate packet loss
        packet_loss_percent = ((total_sent - total_received) / total_sent) * 100
        
        # Calculate RTT statistics
        if successful_results:
            response_times = [r.response_time for r in successful_results]
            min_rtt = min(response_times)
            avg_rtt = statistics.mean(response_times)
            max_rtt = max(response_times)
            
            # Calculate jitter (mean deviation)
            if len(response_times) > 1:
                mdev_rtt = statistics.mean([abs(x - avg_rtt) for x in response_times])
            else:
                mdev_rtt = 0.0
        else:
            min_rtt = None
            avg_rtt = None
            max_rtt = None
            mdev_rtt = None
        
        return PingSummary(
            target=target,
            total_sent=total_sent,
            total_received=total_received,
            packet_loss_percent=packet_loss_percent,
            min_rtt=min_rtt,
            avg_rtt=avg_rtt,
            max_rtt=max_rtt,
            mdev_rtt=mdev_rtt
        )
    
    async def ping_multiple_targets(
        self, 
        targets: List[str], 
        count: int = 4, 
        interval: float = 1.0,
        timeout: float = 5.0
    ) -> List[PingSummary]:
        """Ping multiple targets concurrently and return summaries."""
        tasks = []
        for target in targets:
            # Disable progress for concurrent pings to avoid conflicts
            task = self._ping_target_no_progress(target, count, interval, timeout)
            tasks.append(task)
        
        # Run all ping tasks concurrently
        all_results = await asyncio.gather(*tasks, return_exceptions=True)
        
        summaries = []
        for i, result in enumerate(all_results):
            if isinstance(result, Exception):
                console.print(f"[red]Ping failed for {targets[i]}: {result}[/red]")
                # Create a failed summary
                summaries.append(PingSummary(
                    target=targets[i],
                    total_sent=count,
                    total_received=0,
                    packet_loss_percent=100.0
                ))
            else:
                summary = self.calculate_ping_summary(result)
                summaries.append(summary)
        
        return summaries
    
    async def continuous_ping(
        self, 
        target: str, 
        duration: int = 30, 
        interval: float = 1.0
    ) -> List[PingResult]:
        """Perform continuous ping for a specified duration."""
        results = []
        start_time = datetime.now()
        end_time = start_time.timestamp() + duration
        
        with Progress(
            SpinnerColumn(),
            TextColumn(f"Continuous ping to {target}..."),
            console=console
        ) as progress:
            task = progress.add_task("", total=None)
            
            while datetime.now().timestamp() < end_time:
                # Send a single ping
                ping_results = await self.ping_target(target, count=1, interval=0.1, timeout=3.0)
                if ping_results:
                    results.extend(ping_results)
                
                # Wait for the specified interval
                await asyncio.sleep(interval)
        
        return results


# Convenience functions
async def ping_host(target: str, count: int = 4) -> PingSummary:
    """Ping a single host and return summary."""
    tester = PingTester()
    results = await tester.ping_target(target, count)
    return tester.calculate_ping_summary(results)


async def ping_hosts(targets: List[str], count: int = 4) -> List[PingSummary]:
    """Ping multiple hosts concurrently and return summaries."""
    tester = PingTester()
    return await tester.ping_multiple_targets(targets, count)


def format_ping_summary(summary: PingSummary) -> str:
    """Format ping summary for display."""
    if summary.total_sent == 0:
        return f"No ping data for {summary.target}"
    
    lines = [
        f"Ping to {summary.target}:",
        f"  Packets: {summary.total_received}/{summary.total_sent} received",
        f"  Packet loss: {summary.packet_loss_percent:.1f}%"
    ]
    
    if summary.avg_rtt is not None:
        lines.extend([
            f"  RTT: min={summary.min_rtt:.1f}ms, avg={summary.avg_rtt:.1f}ms, max={summary.max_rtt:.1f}ms",
            f"  Jitter: {summary.mdev_rtt:.1f}ms"
        ])
    
    lines.append(f"  Quality: {summary.quality.value}")
    
    return "\n".join(lines) 