"""System detection and tool availability checking."""

import os
import platform
import shutil
import subprocess
from typing import Dict, List, Optional, Tuple
from pathlib import Path

from rich.console import Console
from rich.panel import Panel

console = Console()


class SystemInfo:
    """System information and available tools."""
    
    def __init__(self):
        self.os_type = self._detect_os()
        self.available_tools = self._detect_tools()
        self.is_root = os.geteuid() == 0 if hasattr(os, 'geteuid') else False
    
    def _detect_os(self) -> str:
        """Detect the operating system."""
        system = platform.system().lower()
        if system == "linux":
            return "linux"
        elif system == "darwin":
            return "macos"
        elif system == "windows":
            return "windows"
        else:
            return "unknown"
    
    def _detect_tools(self) -> Dict[str, bool]:
        """Detect available networking tools."""
        tools = {
            'ping': False,
            'iw': False,
            'iwconfig': False,
            'nmcli': False,
            'airport': False,
            'iperf3': False,
            'curl': False,
            'wget': False,
        }
        
        # Check for basic tools
        for tool in tools:
            if shutil.which(tool):
                tools[tool] = True
        
        # OS-specific tool detection
        if self.os_type == "linux":
            # On Linux, check for wireless tools
            if not tools['iw'] and not tools['iwconfig']:
                console.print("[yellow]Warning: No wireless tools (iw/iwconfig) found. WiFi stats may be limited.[/yellow]")
            
            if not tools['nmcli']:
                console.print("[yellow]Warning: NetworkManager CLI not found. Some features may be limited.[/yellow]")
        
        elif self.os_type == "macos":
            # On macOS, check for airport utility
            airport_path = "/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport"
            if os.path.exists(airport_path):
                tools['airport'] = True
        
        return tools
    
    def get_ping_command(self, target: str, count: int = 4, interval: float = 1.0) -> List[str]:
        """Get the appropriate ping command for the system."""
        if self.os_type == "linux":
            return ["ping", "-c", str(count), "-i", str(interval), "-W", "1", target]
        elif self.os_type == "macos":
            return ["ping", "-c", str(count), "-i", str(interval), "-t", "1", target]
        else:
            # Fallback for other systems
            return ["ping", "-c", str(count), target]
    
    def get_wireless_interface_command(self) -> Optional[List[str]]:
        """Get command to list wireless interfaces."""
        if self.os_type == "linux":
            if self.available_tools['iw']:
                return ["iw", "dev"]
            elif self.available_tools['iwconfig']:
                return ["iwconfig"]
        elif self.os_type == "macos":
            if self.available_tools['airport']:
                return ["/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport", "-I"]
        
        return None
    
    def get_wifi_stats_command(self, interface: str) -> Optional[List[str]]:
        """Get command to retrieve WiFi statistics."""
        if self.os_type == "linux":
            if self.available_tools['iw']:
                return ["iw", "dev", interface, "link"]
            elif self.available_tools['iwconfig']:
                return ["iwconfig", interface]
        elif self.os_type == "macos":
            if self.available_tools['airport']:
                return ["/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport", "-I"]
        
        return None
    
    def get_network_interfaces_command(self) -> Optional[List[str]]:
        """Get command to list network interfaces."""
        if self.os_type == "linux":
            if self.available_tools['nmcli']:
                return ["nmcli", "-t", "-f", "DEVICE,TYPE,STATE", "device", "show"]
            else:
                return ["ip", "link", "show"]
        elif self.os_type == "macos":
            return ["ifconfig"]
        
        return None
    
    def get_iperf3_command(self, server: str, duration: int = 10, direction: str = "download") -> List[str]:
        """Get iperf3 command for throughput testing."""
        cmd = ["iperf3", "-c", server, "-t", str(duration), "-J"]
        
        if direction == "upload":
            cmd.append("-R")
        
        return cmd
    
    def check_requirements(self) -> Tuple[bool, List[str]]:
        """Check if system meets minimum requirements."""
        issues = []
        
        # Check for ping
        if not self.available_tools['ping']:
            issues.append("ping command not found")
        
        # Check for at least one wireless tool
        wireless_tools = ['iw', 'iwconfig', 'airport']
        if not any(self.available_tools[tool] for tool in wireless_tools):
            issues.append("No wireless tools found (iw, iwconfig, or airport)")
        
        # Check for download tool
        download_tools = ['curl', 'wget']
        if not any(self.available_tools[tool] for tool in download_tools):
            issues.append("No download tool found (curl or wget)")
        
        return len(issues) == 0, issues
    
    def print_system_info(self):
        """Print system information and available tools."""
        console.print(Panel(
            f"[bold]System Information[/bold]\n"
            f"OS: {self.os_type.title()}\n"
            f"Root privileges: {'Yes' if self.is_root else 'No'}\n\n"
            f"[bold]Available Tools:[/bold]\n" +
            "\n".join([f"  {tool}: {'✓' if available else '✗'}" 
                      for tool, available in self.available_tools.items()]),
            title="System Detection",
            border_style="blue"
        ))


def get_system_info() -> SystemInfo:
    """Get system information singleton."""
    if not hasattr(get_system_info, '_instance'):
        get_system_info._instance = SystemInfo()
    return get_system_info._instance


def check_network_connectivity(target: str = "8.8.8.8") -> bool:
    """Check basic network connectivity."""
    try:
        system_info = get_system_info()
        cmd = system_info.get_ping_command(target, count=1)
        result = subprocess.run(cmd, capture_output=True, text=True, timeout=5)
        return result.returncode == 0
    except (subprocess.TimeoutExpired, FileNotFoundError):
        return False


def get_wireless_interfaces() -> List[str]:
    """Get list of wireless interface names."""
    system_info = get_system_info()
    interfaces = []
    
    try:
        if system_info.os_type == "linux":
            # Try to get wireless interfaces from /proc/net/wireless
            if os.path.exists("/proc/net/wireless"):
                with open("/proc/net/wireless", "r") as f:
                    lines = f.readlines()[2:]  # Skip header lines
                    for line in lines:
                        if line.strip():
                            parts = line.split()
                            if len(parts) >= 2:
                                interface = parts[0].rstrip(':')
                                interfaces.append(interface)
            
            # Fallback to iw dev
            if not interfaces and system_info.available_tools['iw']:
                result = subprocess.run(["iw", "dev"], capture_output=True, text=True)
                if result.returncode == 0:
                    for line in result.stdout.split('\n'):
                        if 'Interface' in line:
                            parts = line.split()
                            if len(parts) >= 2:
                                interfaces.append(parts[1])
        
        elif system_info.os_type == "macos":
            # On macOS, wireless interfaces typically start with 'en'
            result = subprocess.run(["ifconfig"], capture_output=True, text=True)
            if result.returncode == 0:
                for line in result.stdout.split('\n'):
                    if line.strip() and not line.startswith('\t'):
                        interface = line.split(':')[0]
                        if interface.startswith('en'):
                            interfaces.append(interface)
    
    except Exception as e:
        console.print(f"[red]Error getting wireless interfaces: {e}[/red]")
    
    return list(set(interfaces))  # Remove duplicates 