"""Throughput testing for bandwidth measurements."""

import asyncio
import json
import re
import time
from typing import List, Optional, Tuple
from datetime import datetime
from pathlib import Path

from .models import ThroughputResult
from .system_detect import get_system_info
from ..utils.shell import run_command, ShellError

from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TimeElapsedColumn

console = Console()


class ThroughputTester:
    """Perform throughput tests using various methods."""
    
    def __init__(self):
        self.system_info = get_system_info()
    
    async def test_throughput(
        self,
        direction: str = "download",
        duration: int = 10,
        server: Optional[str] = None,
        url: Optional[str] = None
    ) -> Optional[ThroughputResult]:
        """
        Test throughput using available methods.
        
        Args:
            direction: "download" or "upload"
            duration: Test duration in seconds
            server: iperf3 server (optional)
            url: HTTP download URL (optional)
        
        Returns:
            ThroughputResult or None if test fails
        """
        # Try iperf3 first if available
        if self.system_info.available_tools['iperf3']:
            result = await self._test_with_iperf3(direction, duration, server)
            if result:
                return result
        
        # Fallback to HTTP download test
        if direction == "download":
            return await self._test_http_download(duration, url)
        
        # For upload, we can't easily test without iperf3
        console.print("[yellow]Warning: Upload testing requires iperf3. Skipping upload test.[/yellow]")
        return None
    
    async def _test_with_iperf3(
        self, 
        direction: str, 
        duration: int, 
        server: Optional[str] = None
    ) -> Optional[ThroughputResult]:
        """Test throughput using iperf3."""
        if not server:
            # Use working iperf3 servers
            servers = [
                "iperf.volia.net",
                "iperf.biznetnetworks.com",
                "iperf.steelbytes.com",
                "iperf.tele2.net"
            ]
        else:
            servers = [server]
        
        # Try each server until one works
        for server in servers:
            try:
                cmd = self.system_info.get_iperf3_command(server, duration, direction)
                
                returncode, stdout, stderr = await run_command(
                    cmd,
                    timeout=duration + 10,
                    show_progress=True,
                    progress_text=f"Testing {direction} with iperf3 ({server})..."
                )
                
                if returncode == 0:
                    return self._parse_iperf3_output(stdout, direction, duration, server)
                else:
                    console.print(f"[yellow]iperf3 failed for {server}: {stderr}[/yellow]")
                    continue
                    
            except Exception as e:
                console.print(f"[yellow]iperf3 test failed for {server}: {e}[/yellow]")
                continue
        
        console.print("[red]All iperf3 servers failed[/red]")
        return None
    
    def _parse_iperf3_output(self, output: str, direction: str, duration: int, server: str) -> Optional[ThroughputResult]:
        """Parse iperf3 JSON output."""
        try:
            data = json.loads(output)
            
            # Extract bandwidth information from end section
            if 'end' in data:
                end_data = data['end']
                
                # Try to get bandwidth from sum_sent (for upload) or sum_received (for download)
                if direction == "upload" and 'sum_sent' in end_data:
                    bandwidth_data = end_data['sum_sent']
                elif direction == "download" and 'sum_received' in end_data:
                    bandwidth_data = end_data['sum_received']
                else:
                    # Fallback to streams if sum data not available
                    if 'streams' in end_data:
                        streams = end_data['streams']
                        if streams:
                            # Sum up bandwidth from all streams
                            total_bytes = sum(stream.get('bytes', 0) for stream in streams)
                            total_seconds = streams[0].get('seconds', duration)
                            
                            # Calculate speed in Mbps
                            speed_mbps = (total_bytes * 8) / (total_seconds * 1_000_000)
                            
                            return ThroughputResult(
                                direction=direction,
                                speed_mbps=speed_mbps,
                                duration_seconds=total_seconds,
                                bytes_transferred=total_bytes,
                                test_type="iperf3",
                                target=server
                            )
                    return None
                
                # Extract data from sum_sent/sum_received
                total_bytes = bandwidth_data.get('bytes', 0)
                total_seconds = bandwidth_data.get('seconds', duration)
                bits_per_second = bandwidth_data.get('bits_per_second', 0)
                
                # Calculate speed in Mbps
                speed_mbps = bits_per_second / 1_000_000
                
                return ThroughputResult(
                    direction=direction,
                    speed_mbps=speed_mbps,
                    duration_seconds=total_seconds,
                    bytes_transferred=total_bytes,
                    test_type="iperf3",
                    target=server
                )
            
            # Fallback parsing for non-JSON output
            return self._parse_iperf3_text_output(output, direction, duration, server)
            
        except json.JSONDecodeError:
            # Try parsing text output
            return self._parse_iperf3_text_output(output, direction, duration, server)
        except Exception as e:
            console.print(f"[red]Error parsing iperf3 output: {e}[/red]")
            return None
    
    def _parse_iperf3_text_output(self, output: str, direction: str, duration: int, server: str) -> Optional[ThroughputResult]:
        """Parse iperf3 text output (fallback)."""
        try:
            lines = output.split('\n')
            
            # Look for bandwidth information
            for line in lines:
                line = line.strip()
                
                # Match bandwidth patterns
                # Example: [SUM] 0.00-10.00 sec 1.25 GBytes 1.07 Gbits/sec
                match = re.search(r'(\d+\.\d+)\s*([KMGT]?bits/sec)', line)
                if match:
                    speed_str = match.group(1)
                    unit = match.group(2)
                    
                    speed_mbps = float(speed_str)
                    if unit.startswith('K'):
                        speed_mbps /= 1000
                    elif unit.startswith('G'):
                        speed_mbps *= 1000
                    elif unit.startswith('T'):
                        speed_mbps *= 1_000_000
                    
                    # Estimate bytes transferred
                    bytes_transferred = int((speed_mbps * 1_000_000 * duration) / 8)
                    
                    return ThroughputResult(
                        direction=direction,
                        speed_mbps=speed_mbps,
                        duration_seconds=duration,
                        bytes_transferred=bytes_transferred,
                        test_type="iperf3",
                        target=server
                    )
            
            return None
            
        except Exception as e:
            console.print(f"[red]Error parsing iperf3 text output: {e}[/red]")
            return None
    
    async def _test_http_download(self, duration: int, url: Optional[str] = None) -> Optional[ThroughputResult]:
        """Test download speed using HTTP download."""
        if not url:
            # Use a reliable download URL
            url = "https://httpbin.org/bytes/10485760"  # 10MB file
        
        try:
            # Determine download tool
            if self.system_info.available_tools['curl']:
                return await self._test_http_download_curl(url, duration)
            elif self.system_info.available_tools['wget']:
                return await self._test_http_download_wget(url, duration)
            else:
                console.print("[red]No download tool available (curl or wget)[/red]")
                return None
                
        except Exception as e:
            console.print(f"[red]HTTP download test failed: {e}[/red]")
            return None
    
    async def _test_http_download_curl(self, url: str, duration: int) -> Optional[ThroughputResult]:
        """Test download speed using curl."""
        try:
            # Create a temporary file for download
            temp_file = Path(f"/tmp/wifi_profiler_download_{int(time.time())}")
            
            # Start download with progress monitoring
            start_time = time.time()
            
            cmd = [
                "curl", "-o", str(temp_file), "-k", "-s", "-w", 
                "time_total:%{time_total},size_download:%{size_download},speed_download:%{speed_download}",
                url
            ]
            
            returncode, stdout, stderr = await run_command(
                cmd,
                timeout=duration + 30,
                show_progress=True,
                progress_text="Testing download speed..."
            )
            
            end_time = time.time()
            actual_duration = end_time - start_time
            
            # Clean up temp file
            try:
                temp_file.unlink(missing_ok=True)
            except Exception:
                pass
            
            if returncode == 0:
                # Parse curl output
                metrics = {}
                for part in stdout.split(','):
                    if ':' in part:
                        key, value = part.split(':', 1)
                        metrics[key] = value
                
                # Extract metrics
                total_time = float(metrics.get('time_total', actual_duration))
                bytes_downloaded = int(metrics.get('size_download', 0))
                speed_bps = float(metrics.get('speed_download', 0))
                
                # Convert to Mbps
                speed_mbps = (speed_bps * 8) / 1_000_000
                
                return ThroughputResult(
                    direction="download",
                    speed_mbps=speed_mbps,
                    duration_seconds=total_time,
                    bytes_transferred=bytes_downloaded,
                    test_type="http_download",
                    target=url
                )
            else:
                console.print(f"[red]curl download failed: {stderr}[/red]")
                return None
                
        except Exception as e:
            console.print(f"[red]curl download test failed: {e}[/red]")
            return None
    
    async def _test_http_download_wget(self, url: str, duration: int) -> Optional[ThroughputResult]:
        """Test download speed using wget."""
        try:
            # Create a temporary file for download
            temp_file = Path(f"/tmp/wifi_profiler_download_{int(time.time())}")
            
            # Start download
            start_time = time.time()
            
            cmd = ["wget", "-O", str(temp_file), "--progress=bar:force", url]
            
            returncode, stdout, stderr = await run_command(
                cmd,
                timeout=duration + 30,
                show_progress=True,
                progress_text="Testing download speed..."
            )
            
            end_time = time.time()
            actual_duration = end_time - start_time
            
            # Clean up temp file
            try:
                temp_file.unlink(missing_ok=True)
            except Exception:
                pass
            
            if returncode == 0:
                # Get file size from wget output
                size_match = re.search(r'\(([\d.]+)\s*([KMGT]?B)\)', stdout)
                if size_match:
                    size_str = size_match.group(1)
                    unit = size_match.group(2)
                    
                    bytes_downloaded = float(size_str)
                    if unit == 'KB':
                        bytes_downloaded *= 1024
                    elif unit == 'MB':
                        bytes_downloaded *= 1024 * 1024
                    elif unit == 'GB':
                        bytes_downloaded *= 1024 * 1024 * 1024
                    
                    # Calculate speed in Mbps
                    speed_mbps = (bytes_downloaded * 8) / (actual_duration * 1_000_000)
                    
                    return ThroughputResult(
                        direction="download",
                        speed_mbps=speed_mbps,
                        duration_seconds=actual_duration,
                        bytes_transferred=int(bytes_downloaded),
                        test_type="http_download",
                        target=url
                    )
            
            return None
            
        except Exception as e:
            console.print(f"[red]wget download test failed: {e}[/red]")
            return None
    
    async def test_multiple_directions(
        self,
        directions: List[str] = None,
        duration: int = 10,
        server: Optional[str] = None,
        url: Optional[str] = None
    ) -> List[ThroughputResult]:
        """Test throughput in multiple directions."""
        if directions is None:
            directions = ["download"]
            if self.system_info.available_tools['iperf3']:
                directions.append("upload")
        
        results = []
        
        for direction in directions:
            result = await self.test_throughput(direction, duration, server, url)
            if result:
                results.append(result)
        
        return results


# Convenience functions
async def test_download_speed(duration: int = 10, url: Optional[str] = None) -> Optional[ThroughputResult]:
    """Test download speed."""
    tester = ThroughputTester()
    return await tester.test_throughput("download", duration, url=url)


async def test_upload_speed(duration: int = 10, server: Optional[str] = None) -> Optional[ThroughputResult]:
    """Test upload speed."""
    tester = ThroughputTester()
    return await tester.test_throughput("upload", duration, server=server)


async def test_bandwidth(duration: int = 10) -> List[ThroughputResult]:
    """Test both download and upload speeds."""
    tester = ThroughputTester()
    return await tester.test_multiple_directions(duration=duration)


def format_throughput_result(result: ThroughputResult) -> str:
    """Format throughput result for display."""
    lines = [
        f"{result.direction.title()} Test:",
        f"  Speed: {result.speed_mbps_formatted}",
        f"  Duration: {result.duration_seconds:.1f}s",
        f"  Data: {result.bytes_transferred:,} bytes",
        f"  Method: {result.test_type}"
    ]
    
    if result.target:
        lines.append(f"  Target: {result.target}")
    
    return "\n".join(lines) 