"""WiFi statistics collection and parsing."""

import re
import asyncio
from typing import Dict, List, Optional, Tuple
from datetime import datetime

from .models import InterfaceInfo, WiFiStats
from .system_detect import get_system_info
from ..utils.shell import run_command, ShellError

from rich.console import Console

console = Console()


class WiFiStatsCollector:
    """Collect and parse WiFi statistics from system commands."""
    
    def __init__(self):
        self.system_info = get_system_info()
    
    async def get_interface_info(self, interface: str) -> Optional[InterfaceInfo]:
        """Get detailed information about a wireless interface."""
        try:
            if self.system_info.os_type == "linux":
                return await self._get_linux_interface_info(interface)
            elif self.system_info.os_type == "macos":
                return await self._get_macos_interface_info(interface)
            else:
                return None
        except Exception as e:
            console.print(f"[red]Error getting interface info for {interface}: {e}[/red]")
            return None
    
    async def _get_linux_interface_info(self, interface: str) -> Optional[InterfaceInfo]:
        """Get interface information on Linux."""
        info = InterfaceInfo(name=interface)
        
        # Try iw dev interface link first
        if self.system_info.available_tools['iw']:
            try:
                cmd = ["iw", "dev", interface, "link"]
                returncode, stdout, stderr = await run_command(cmd, timeout=5)
                
                if returncode == 0:
                    # Parse iw output
                    lines = stdout.split('\n')
                    for line in lines:
                        line = line.strip()
                        if 'SSID:' in line:
                            info.ssid = line.split('SSID:')[1].strip()
                        elif 'freq:' in line:
                            freq_match = re.search(r'freq:\s*(\d+)', line)
                            if freq_match:
                                freq_mhz = int(freq_match.group(1))
                                info.frequency = freq_mhz / 1000  # Convert to GHz
                        elif 'tx bitrate:' in line:
                            bitrate_match = re.search(r'tx bitrate:\s*([\d.]+)\s*Mbps', line)
                            if bitrate_match:
                                info.bitrate = float(bitrate_match.group(1))
                        elif 'BSS' in line and 'on freq' in line:
                            bss_match = re.search(r'BSS\s+([a-fA-F0-9:]+)', line)
                            if bss_match:
                                info.bssid = bss_match.group(1)
                    
                    info.is_connected = bool(info.ssid)
                    return info
            except Exception:
                pass
        
        # Fallback to iwconfig
        if self.system_info.available_tools['iwconfig']:
            try:
                cmd = ["iwconfig", interface]
                returncode, stdout, stderr = await run_command(cmd, timeout=5)
                
                if returncode == 0:
                    # Parse iwconfig output
                    lines = stdout.split('\n')
                    for line in lines:
                        line = line.strip()
                        if 'ESSID:' in line:
                            ssid_match = re.search(r'ESSID:"([^"]*)"', line)
                            if ssid_match:
                                info.ssid = ssid_match.group(1)
                        elif 'Frequency=' in line:
                            freq_match = re.search(r'Frequency=([\d.]+)\s*GHz', line)
                            if freq_match:
                                info.frequency = float(freq_match.group(1))
                        elif 'Bit Rate=' in line:
                            bitrate_match = re.search(r'Bit Rate=([\d.]+)\s*Mb/s', line)
                            if bitrate_match:
                                info.bitrate = float(bitrate_match.group(1))
                        elif 'Access Point:' in line:
                            ap_match = re.search(r'Access Point:\s*([a-fA-F0-9:]+)', line)
                            if ap_match:
                                info.bssid = ap_match.group(1)
                    
                    info.is_connected = bool(info.ssid)
                    return info
            except Exception:
                pass
        
        return info
    
    async def _get_macos_interface_info(self, interface: str) -> Optional[InterfaceInfo]:
        """Get interface information on macOS."""
        info = InterfaceInfo(name=interface)
        
        if self.system_info.available_tools['airport']:
            try:
                cmd = ["/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport", "-I"]
                returncode, stdout, stderr = await run_command(cmd, timeout=5)
                
                if returncode == 0:
                    lines = stdout.split('\n')
                    for line in lines:
                        line = line.strip()
                        if ' SSID: ' in line:
                            info.ssid = line.split(' SSID: ')[1].strip()
                        elif ' BSSID: ' in line:
                            info.bssid = line.split(' BSSID: ')[1].strip()
                        elif ' channel: ' in line:
                            channel_match = re.search(r' channel: (\d+)', line)
                            if channel_match:
                                info.channel = int(channel_match.group(1))
                        elif ' lastTxRate: ' in line:
                            rate_match = re.search(r' lastTxRate: ([\d.]+)', line)
                            if rate_match:
                                info.bitrate = float(rate_match.group(1))
                    
                    info.is_connected = bool(info.ssid)
                    return info
            except Exception:
                pass
        
        return info
    
    async def get_wifi_stats(self, interface: str) -> Optional[WiFiStats]:
        """Get current WiFi statistics for an interface."""
        try:
            if self.system_info.os_type == "linux":
                return await self._get_linux_wifi_stats(interface)
            elif self.system_info.os_type == "macos":
                return await self._get_macos_wifi_stats(interface)
            else:
                return None
        except Exception as e:
            console.print(f"[red]Error getting WiFi stats for {interface}: {e}[/red]")
            return None
    
    async def _get_linux_wifi_stats(self, interface: str) -> Optional[WiFiStats]:
        """Get WiFi statistics on Linux."""
        stats = WiFiStats()
        
        # Try to get stats from /proc/net/wireless
        try:
            with open('/proc/net/wireless', 'r') as f:
                lines = f.readlines()
                for line in lines[2:]:  # Skip header lines
                    if interface in line:
                        parts = line.split()
                        if len(parts) >= 4:
                            # Format: Interface | Status | Link | Level | Noise | ...
                            # Level is signal strength, Noise is noise level
                            try:
                                level = int(parts[2])
                                noise = int(parts[3])
                                
                                # Convert to dBm (approximate conversion)
                                stats.rssi = level - 256
                                stats.noise = noise - 256
                                
                                # Calculate signal quality
                                if stats.rssi is not None and stats.noise is not None:
                                    snr = stats.rssi - stats.noise
                                    stats.signal_quality = max(0, min(100, (snr + 20) * 2.5))
                            except (ValueError, IndexError):
                                pass
                        break
        except Exception:
            pass
        
        # Try iw dev interface link for additional stats
        if self.system_info.available_tools['iw']:
            try:
                cmd = ["iw", "dev", interface, "link"]
                returncode, stdout, stderr = await run_command(cmd, timeout=5)
                
                if returncode == 0:
                    lines = stdout.split('\n')
                    for line in lines:
                        line = line.strip()
                        if 'signal:' in line:
                            signal_match = re.search(r'signal:\s*(-?\d+)\s*dBm', line)
                            if signal_match:
                                stats.rssi = int(signal_match.group(1))
                        elif 'tx bitrate:' in line:
                            bitrate_match = re.search(r'tx bitrate:\s*([\d.]+)\s*Mbps', line)
                            if bitrate_match:
                                # This is already in the interface info, but good to have here too
                                pass
            except Exception:
                pass
        
        return stats
    
    async def _get_macos_wifi_stats(self, interface: str) -> Optional[WiFiStats]:
        """Get WiFi statistics on macOS."""
        stats = WiFiStats()
        
        if self.system_info.available_tools['airport']:
            try:
                cmd = ["/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport", "-I"]
                returncode, stdout, stderr = await run_command(cmd, timeout=5)
                
                if returncode == 0:
                    lines = stdout.split('\n')
                    for line in lines:
                        line = line.strip()
                        if ' agrCtlRSSI: ' in line:
                            rssi_match = re.search(r' agrCtlRSSI: (-?\d+)', line)
                            if rssi_match:
                                stats.rssi = int(rssi_match.group(1))
                        elif ' agrCtlNoise: ' in line:
                            noise_match = re.search(r' agrCtlNoise: (-?\d+)', line)
                            if noise_match:
                                stats.noise = int(noise_match.group(1))
                        elif ' lastTxRate: ' in line:
                            rate_match = re.search(r' lastTxRate: ([\d.]+)', line)
                            if rate_match:
                                # This is already in interface info
                                pass
                    
                    # Calculate signal quality if we have RSSI
                    if stats.rssi is not None:
                        # Rough conversion: -50 dBm = 100%, -100 dBm = 0%
                        stats.signal_quality = max(0, min(100, (stats.rssi + 100) * 2))
            except Exception:
                pass
        
        return stats
    
    async def get_all_interfaces(self) -> List[InterfaceInfo]:
        """Get information about all wireless interfaces."""
        interfaces = []
        
        # Get list of wireless interfaces
        wireless_interfaces = []
        
        if self.system_info.os_type == "linux":
            # Try /proc/net/wireless first
            try:
                with open('/proc/net/wireless', 'r') as f:
                    lines = f.readlines()[2:]  # Skip header
                    for line in lines:
                        if line.strip():
                            parts = line.split()
                            if len(parts) >= 2:
                                interface = parts[0].rstrip(':')
                                wireless_interfaces.append(interface)
            except Exception:
                pass
            
            # Fallback to iw dev
            if not wireless_interfaces and self.system_info.available_tools['iw']:
                try:
                    cmd = ["iw", "dev"]
                    returncode, stdout, stderr = await run_command(cmd, timeout=5)
                    if returncode == 0:
                        for line in stdout.split('\n'):
                            if 'Interface' in line:
                                parts = line.split()
                                if len(parts) >= 2:
                                    wireless_interfaces.append(parts[1])
                except Exception:
                    pass
        
        elif self.system_info.os_type == "macos":
            # On macOS, wireless interfaces typically start with 'en'
            try:
                cmd = ["ifconfig"]
                returncode, stdout, stderr = await run_command(cmd, timeout=5)
                if returncode == 0:
                    for line in stdout.split('\n'):
                        if line.strip() and not line.startswith('\t'):
                            interface = line.split(':')[0]
                            if interface.startswith('en'):
                                wireless_interfaces.append(interface)
            except Exception:
                pass
        
        # Get detailed info for each interface
        for interface in wireless_interfaces:
            info = await self.get_interface_info(interface)
            if info:
                interfaces.append(info)
        
        return interfaces
    
    async def monitor_wifi_stats(
        self, 
        interface: str, 
        duration: int = 30, 
        interval: float = 1.0
    ) -> List[WiFiStats]:
        """Monitor WiFi stats over time."""
        stats_list = []
        start_time = datetime.now()
        end_time = start_time.timestamp() + duration
        
        while datetime.now().timestamp() < end_time:
            stats = await self.get_wifi_stats(interface)
            if stats:
                stats_list.append(stats)
            
            await asyncio.sleep(interval)
        
        return stats_list


# Convenience functions
async def get_current_wifi_stats(interface: str) -> Optional[WiFiStats]:
    """Get current WiFi statistics for an interface."""
    collector = WiFiStatsCollector()
    return await collector.get_wifi_stats(interface)


async def get_interface_details(interface: str) -> Optional[InterfaceInfo]:
    """Get detailed interface information."""
    collector = WiFiStatsCollector()
    return await collector.get_interface_info(interface)


async def list_wireless_interfaces() -> List[InterfaceInfo]:
    """List all wireless interfaces with details."""
    collector = WiFiStatsCollector()
    return await collector.get_all_interfaces() 