"""HTML report generation for WiFi profiler results."""

import json
from datetime import datetime
from pathlib import Path
from typing import Dict, List, Optional

import plotly.graph_objects as go
import plotly.express as px
from plotly.subplots import make_subplots
from jinja2 import Template

from .core.models import TestSession, ConnectionQuality


# HTML template for the report
HTML_TEMPLATE = """
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>WiFi Profiler Report - {{ session.session_id }}</title>
    <script src="https://cdn.plot.ly/plotly-latest.min.js"></script>
    <style>
        body {
            font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif;
            line-height: 1.6;
            margin: 0;
            padding: 20px;
            background-color: #f5f5f5;
        }
        .container {
            max-width: 1200px;
            margin: 0 auto;
            background: white;
            padding: 30px;
            border-radius: 10px;
            box-shadow: 0 2px 10px rgba(0,0,0,0.1);
        }
        .header {
            text-align: center;
            margin-bottom: 30px;
            padding-bottom: 20px;
            border-bottom: 2px solid #eee;
        }
        .header h1 {
            color: #333;
            margin: 0;
            font-size: 2.5em;
        }
        .header .subtitle {
            color: #666;
            font-size: 1.2em;
            margin-top: 10px;
        }
        .summary-grid {
            display: grid;
            grid-template-columns: repeat(auto-fit, minmax(250px, 1fr));
            gap: 20px;
            margin-bottom: 30px;
        }
        .summary-card {
            background: #f8f9fa;
            padding: 20px;
            border-radius: 8px;
            border-left: 4px solid #007bff;
        }
        .summary-card h3 {
            margin: 0 0 10px 0;
            color: #333;
        }
        .summary-card .value {
            font-size: 1.5em;
            font-weight: bold;
            color: #007bff;
        }
        .quality-excellent { color: #28a745; }
        .quality-good { color: #17a2b8; }
        .quality-fair { color: #ffc107; }
        .quality-poor { color: #dc3545; }
        .quality-unknown { color: #6c757d; }
        .chart-container {
            margin: 30px 0;
            padding: 20px;
            background: #f8f9fa;
            border-radius: 8px;
        }
        .chart-container h3 {
            margin-top: 0;
            color: #333;
        }
        .ping-table {
            width: 100%;
            border-collapse: collapse;
            margin: 20px 0;
        }
        .ping-table th,
        .ping-table td {
            padding: 12px;
            text-align: left;
            border-bottom: 1px solid #ddd;
        }
        .ping-table th {
            background-color: #f8f9fa;
            font-weight: bold;
        }
        .ping-table tr:hover {
            background-color: #f5f5f5;
        }
        .throughput-results {
            display: grid;
            grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
            gap: 20px;
            margin: 20px 0;
        }
        .throughput-card {
            background: #f8f9fa;
            padding: 20px;
            border-radius: 8px;
            border-left: 4px solid #28a745;
        }
        .footer {
            text-align: center;
            margin-top: 40px;
            padding-top: 20px;
            border-top: 1px solid #eee;
            color: #666;
        }
        @media (max-width: 768px) {
            .container {
                padding: 15px;
            }
            .summary-grid {
                grid-template-columns: 1fr;
            }
        }
    </style>
</head>
<body>
    <div class="container">
        <div class="header">
            <h1>WiFi Profiler Report</h1>
            <div class="subtitle">
                Session: {{ session.session_id }} | 
                {{ session.start_time.strftime('%Y-%m-%d %H:%M:%S') }}
            </div>
        </div>

        <!-- Summary Section -->
        <div class="summary-grid">
            <div class="summary-card">
                <h3>Interface</h3>
                <div class="value">{{ session.interface.name if session.interface else 'Unknown' }}</div>
            </div>
            <div class="summary-card">
                <h3>SSID</h3>
                <div class="value">{{ session.interface.ssid if session.interface and session.interface.ssid else 'Not connected' }}</div>
            </div>
            <div class="summary-card">
                <h3>Duration</h3>
                <div class="value">{{ "%.1f"|format(session.duration) if session.duration else 0 }}s</div>
            </div>
            <div class="summary-card">
                <h3>Overall Quality</h3>
                <div class="value quality-{{ overall_quality.value }}">{{ overall_quality.value.title() }}</div>
            </div>
        </div>

        <!-- WiFi Statistics Chart -->
        {% if session.wifi_stats %}
        <div class="chart-container">
            <h3>WiFi Signal Strength Over Time</h3>
            <div id="rssi-chart"></div>
        </div>
        {% endif %}

        <!-- Ping Results -->
        {% if session.ping_summaries %}
        <div class="chart-container">
            <h3>Ping Test Results</h3>
            <table class="ping-table">
                <thead>
                    <tr>
                        <th>Target</th>
                        <th>Avg RTT (ms)</th>
                        <th>Packet Loss (%)</th>
                        <th>Quality</th>
                        <th>Success Rate (%)</th>
                    </tr>
                </thead>
                <tbody>
                    {% for summary in session.ping_summaries %}
                    <tr>
                        <td>{{ summary.target }}</td>
                        <td>{{ "%.1f"|format(summary.avg_rtt) if summary.avg_rtt else 'N/A' }}</td>
                        <td>{{ "%.1f"|format(summary.packet_loss_percent) }}</td>
                        <td class="quality-{{ summary.quality.value }}">{{ summary.quality.value.title() }}</td>
                        <td>{{ "%.1f"|format(summary.success_rate) }}</td>
                    </tr>
                    {% endfor %}
                </tbody>
            </table>
        </div>
        {% endif %}

        <!-- Throughput Results -->
        {% if session.throughput_results %}
        <div class="chart-container">
            <h3>Throughput Test Results</h3>
            <div class="throughput-results">
                {% for result in session.throughput_results %}
                <div class="throughput-card">
                    <h4>{{ result.direction.title() }} Speed</h4>
                    <div class="value">{{ result.speed_mbps_formatted }}</div>
                    <p><strong>Duration:</strong> {{ "%.1f"|format(result.duration_seconds) }}s</p>
                    <p><strong>Data Transferred:</strong> {{ "{:,}".format(result.bytes_transferred) }} bytes</p>
                    <p><strong>Method:</strong> {{ result.test_type }}</p>
                    {% if result.target %}
                    <p><strong>Target:</strong> {{ result.target }}</p>
                    {% endif %}
                </div>
                {% endfor %}
            </div>
        </div>
        {% endif %}

        <!-- Configuration -->
        <div class="chart-container">
            <h3>Test Configuration</h3>
            <table class="ping-table">
                <tbody>
                    {% for key, value in session.config.items() %}
                    <tr>
                        <td><strong>{{ key.replace('_', ' ').title() }}</strong></td>
                        <td>{{ value }}</td>
                    </tr>
                    {% endfor %}
                </tbody>
            </table>
        </div>

        <div class="footer">
            <p>Generated by WiFi Profiler on {{ datetime.now().strftime('%Y-%m-%d %H:%M:%S') }}</p>
        </div>
    </div>

    <script>
        // RSSI Chart
        {% if session.wifi_stats %}
        const rssiData = {{ rssi_chart_data | tojson }};
        Plotly.newPlot('rssi-chart', rssiData.data, rssiData.layout);
        {% endif %}
    </script>
</body>
</html>
"""


def generate_html_report(session: TestSession, output_path: Path) -> None:
    """Generate an HTML report from a test session."""
    
    # Calculate overall quality
    overall_quality = _calculate_overall_quality(session)
    
    # Prepare RSSI chart data
    rssi_chart_data = _prepare_rssi_chart(session)
    
    # Convert Plotly objects to JSON for template
    import plotly.utils
    rssi_chart_json = plotly.utils.PlotlyJSONEncoder().encode(rssi_chart_data)
    
    # Render template
    template = Template(HTML_TEMPLATE)
    html_content = template.render(
        session=session,
        overall_quality=overall_quality,
        rssi_chart_data=rssi_chart_json,
        datetime=datetime
    )
    
    # Write to file
    with open(output_path, 'w', encoding='utf-8') as f:
        f.write(html_content)


def _calculate_overall_quality(session: TestSession) -> ConnectionQuality:
    """Calculate overall connection quality from session data."""
    if not session.ping_summaries:
        return ConnectionQuality.UNKNOWN
    
    # Calculate average packet loss and latency
    avg_packet_loss = sum(s.packet_loss_percent for s in session.ping_summaries) / len(session.ping_summaries)
    
    avg_latency = 0
    latency_count = 0
    for summary in session.ping_summaries:
        if summary.avg_rtt:
            avg_latency += summary.avg_rtt
            latency_count += 1
    
    if latency_count > 0:
        avg_latency /= latency_count
    
    # Determine quality based on thresholds
    if avg_packet_loss > 10:
        return ConnectionQuality.POOR
    elif avg_packet_loss > 5 or avg_latency > 100:
        return ConnectionQuality.FAIR
    elif avg_packet_loss < 1 and avg_latency < 50:
        return ConnectionQuality.EXCELLENT
    else:
        return ConnectionQuality.GOOD


def _prepare_rssi_chart(session: TestSession) -> Dict:
    """Prepare RSSI chart data for Plotly."""
    if not session.wifi_stats:
        return {"data": [], "layout": {}}
    
    # Extract RSSI data
    timestamps = []
    rssi_values = []
    
    for stats in session.wifi_stats:
        if stats.rssi is not None:
            timestamps.append(stats.timestamp)
            rssi_values.append(stats.rssi)
    
    if not rssi_values:
        return {"data": [], "layout": {}}
    
    # Create trace
    trace = go.Scatter(
        x=timestamps,
        y=rssi_values,
        mode='lines+markers',
        name='RSSI',
        line=dict(color='#007bff', width=2),
        marker=dict(size=6, color='#007bff')
    )
    
    # Create layout
    layout = go.Layout(
        title='WiFi Signal Strength Over Time',
        xaxis=dict(
            title='Time',
            showgrid=True,
            gridcolor='#e1e5e9'
        ),
        yaxis=dict(
            title='RSSI (dBm)',
            showgrid=True,
            gridcolor='#e1e5e9',
            range=[min(rssi_values) - 5, max(rssi_values) + 5]
        ),
        plot_bgcolor='white',
        paper_bgcolor='white',
        font=dict(size=12),
        margin=dict(l=60, r=30, t=40, b=60)
    )
    
    return {
        "data": [trace],
        "layout": layout
    }


def generate_json_report(session: TestSession, output_path: Path) -> None:
    """Generate a JSON report from a test session."""
    # Convert session to dict with datetime serialization
    session_dict = session.model_dump()
    
    # Add overall quality
    session_dict['overall_quality'] = _calculate_overall_quality(session).value
    
    # Write to file
    with open(output_path, 'w', encoding='utf-8') as f:
        json.dump(session_dict, f, indent=2, default=str)


def generate_summary_report(sessions: List[TestSession], output_path: Path) -> None:
    """Generate a summary report from multiple sessions."""
    
    # Calculate summary statistics
    summary_data = {
        "total_sessions": len(sessions),
        "date_range": {
            "start": min(s.start_time for s in sessions).isoformat(),
            "end": max(s.end_time for s in sessions if s.end_time).isoformat()
        },
        "interfaces": list(set(s.interface.name for s in sessions if s.interface)),
        "overall_qualities": {},
        "avg_rssi": None,
        "avg_latency": None,
        "avg_packet_loss": None
    }
    
    # Calculate quality distribution
    for session in sessions:
        quality = _calculate_overall_quality(session)
        summary_data["overall_qualities"][quality.value] = \
            summary_data["overall_qualities"].get(quality.value, 0) + 1
    
    # Calculate averages
    rssi_values = []
    latency_values = []
    packet_loss_values = []
    
    for session in sessions:
        # RSSI
        for stats in session.wifi_stats:
            if stats.rssi is not None:
                rssi_values.append(stats.rssi)
        
        # Latency and packet loss
        for summary in session.ping_summaries:
            if summary.avg_rtt:
                latency_values.append(summary.avg_rtt)
            packet_loss_values.append(summary.packet_loss_percent)
    
    if rssi_values:
        summary_data["avg_rssi"] = sum(rssi_values) / len(rssi_values)
    if latency_values:
        summary_data["avg_latency"] = sum(latency_values) / len(latency_values)
    if packet_loss_values:
        summary_data["avg_packet_loss"] = sum(packet_loss_values) / len(packet_loss_values)
    
    # Write to file
    with open(output_path, 'w', encoding='utf-8') as f:
        json.dump(summary_data, f, indent=2, default=str) 