"""Live TUI dashboard for real-time WiFi monitoring."""

import asyncio
from datetime import datetime, timedelta
from typing import List, Dict, Optional
from collections import deque

from textual.app import App, ComposeResult
from textual.containers import Container, Horizontal, Vertical
from textual.widgets import (
    Header, Footer, Static, DataTable, Sparkline, 
    Label, ProgressBar, Button, Log
)
from textual.widgets.data_table import RowKey
from textual.reactive import reactive
from textual import work

from ..core.models import WiFiStats, PingResult, PingSummary
from ..core.wifi_stats import WiFiStatsCollector
from ..core.ping import PingTester
from ..core.system_detect import get_system_info

from rich.text import Text


class WiFiStatsWidget(Static):
    """Widget to display current WiFi statistics."""
    
    rssi = reactive(0.0)
    signal_quality = reactive(0.0)
    bitrate = reactive(0.0)
    ssid = reactive("")
    
    def __init__(self, interface: str):
        super().__init__()
        self.interface = interface
        self.collector = WiFiStatsCollector()
    
    def compose(self) -> ComposeResult:
        yield Label("WiFi Statistics", classes="widget-title")
        yield Label("RSSI: -- dBm", id="rssi-label")
        yield Label("Signal Quality: --%", id="signal-quality-label")
        yield Label("Bitrate: -- Mbps", id="bitrate-label")
        yield Label("SSID: --", id="ssid-label")
    
    def watch_rssi(self, rssi: float) -> None:
        """Update RSSI display."""
        if rssi != 0.0:
            self.query_one("#rssi-label").update(f"RSSI: {rssi:.1f} dBm")
        else:
            self.query_one("#rssi-label").update("RSSI: -- dBm")
    
    def watch_signal_quality(self, quality: float) -> None:
        """Update signal quality display."""
        if quality != 0.0:
            self.query_one("#signal-quality-label").update(f"Signal Quality: {quality:.1f}%")
        else:
            self.query_one("#signal-quality-label").update("Signal Quality: --%")
    
    def watch_bitrate(self, bitrate: float) -> None:
        """Update bitrate display."""
        if bitrate != 0.0:
            self.query_one("#bitrate-label").update(f"Bitrate: {bitrate:.1f} Mbps")
        else:
            self.query_one("#bitrate-label").update("Bitrate: -- Mbps")
    
    def watch_ssid(self, ssid: str) -> None:
        """Update SSID display."""
        if ssid:
            self.query_one("#ssid-label").update(f"SSID: {ssid}")
        else:
            self.query_one("#ssid-label").update("SSID: --")
    
    async def update_stats(self) -> None:
        """Update WiFi statistics."""
        try:
            stats = await self.collector.get_wifi_stats(self.interface)
            if stats:
                self.rssi = stats.rssi
                self.signal_quality = stats.signal_quality
                
            # Get interface info for bitrate and SSID
            info = await self.collector.get_interface_info(self.interface)
            if info:
                self.bitrate = info.bitrate
                self.ssid = info.ssid
        except Exception:
            pass


class PingTableWidget(DataTable):
    """Widget to display ping results in a table."""
    
    def __init__(self, targets: List[str]):
        super().__init__()
        self.targets = targets
        self.ping_tester = PingTester()
        self.results: Dict[str, PingSummary] = {}
    
    def on_mount(self) -> None:
        """Initialize the table."""
        self.add_columns("Target", "Avg RTT", "Packet Loss", "Quality", "Status")
        
        # Add rows for each target
        for target in self.targets:
            self.add_row(target, "--", "--", "--", "Waiting...")
    
    async def update_ping_results(self) -> None:
        """Update ping results."""
        try:
            summaries = await self.ping_tester.ping_multiple_targets(self.targets, count=3)
            
            for i, summary in enumerate(summaries):
                if i < len(self.targets):
                    target = self.targets[i]
                    self.results[target] = summary
                    
                    # Update table row
                    avg_rtt = f"{summary.avg_rtt:.1f}ms" if summary.avg_rtt else "--"
                    packet_loss = f"{summary.packet_loss_percent:.1f}%"
                    quality = summary.quality.value
                    status = "✓" if summary.total_received > 0 else "✗"
                    
                    self.update_cell(RowKey(i), "Avg RTT", avg_rtt)
                    self.update_cell(RowKey(i), "Packet Loss", packet_loss)
                    self.update_cell(RowKey(i), "Quality", quality)
                    self.update_cell(RowKey(i), "Status", status)
        except Exception:
            pass


class RSSISparklineWidget(Sparkline):
    """Widget to display RSSI history as a sparkline."""
    
    def __init__(self):
        super().__init__()
        self.data = deque(maxlen=100)  # Keep last 100 values
    
    def add_value(self, value: float) -> None:
        """Add a new RSSI value."""
        self.data.append(value)
        self.data = list(self.data)  # Update sparkline


class LiveViewApp(App):
    """Main live view application."""
    
    CSS = """
    .widget-title {
        text-align: center;
        text-style: bold;
        margin-bottom: 1;
    }
    
    .stats-container {
        height: 20%;
        border: solid green;
    }
    
    .ping-container {
        height: 40%;
        border: solid blue;
    }
    
    .chart-container {
        height: 30%;
        border: solid red;
    }
    
    .control-container {
        height: 10%;
        border: solid yellow;
    }
    """
    
    def __init__(
        self,
        interface: str,
        targets: List[str],
        duration: int = 300,
        interval: float = 1.0
    ):
        super().__init__()
        self.interface = interface
        self.targets = targets
        self.duration = duration
        self.interval = interval
        self.start_time = datetime.now()
        self.running = True
        
        # Initialize widgets
        self.wifi_stats_widget = WiFiStatsWidget(interface)
        self.ping_table_widget = PingTableWidget(targets)
        self.rssi_sparkline = RSSISparklineWidget()
        self.log_widget = Log()
    
    def compose(self) -> ComposeResult:
        """Compose the application layout."""
        yield Header()
        
        with Container():
            with Vertical():
                with Horizontal():
                    yield self.wifi_stats_widget
                    yield self.rssi_sparkline
                
                yield self.ping_table_widget
                
                yield self.log_widget
                
                with Horizontal():
                    yield Button("Save Snapshot", id="save-btn")
                    yield Button("Quit", id="quit-btn")
        
        yield Footer()
    
    def on_mount(self) -> None:
        """Start the monitoring when the app mounts."""
        self.start_monitoring()
    
    def on_button_pressed(self, event: Button.Pressed) -> None:
        """Handle button presses."""
        if event.button.id == "save-btn":
            self.save_snapshot()
        elif event.button.id == "quit-btn":
            self.exit()
    
    def on_key(self, event) -> None:
        """Handle key presses."""
        if event.key == "s":
            self.save_snapshot()
        elif event.key == "q":
            self.exit()
    
    @work
    async def start_monitoring(self) -> None:
        """Start the monitoring loop."""
        self.log_widget.write("Starting WiFi monitoring...")
        
        while self.running:
            try:
                # Update WiFi stats
                await self.wifi_stats_widget.update_stats()
                
                # Add RSSI to sparkline if available
                if self.wifi_stats_widget.rssi is not None:
                    self.rssi_sparkline.add_value(self.wifi_stats_widget.rssi)
                
                # Update ping results
                await self.ping_table_widget.update_ping_results()
                
                # Check if duration has elapsed
                elapsed = (datetime.now() - self.start_time).total_seconds()
                if elapsed >= self.duration:
                    self.log_widget.write("Monitoring duration completed.")
                    break
                
                await asyncio.sleep(self.interval)
                
            except Exception as e:
                self.log_widget.write(f"Error: {e}")
                await asyncio.sleep(self.interval)
    
    def save_snapshot(self) -> None:
        """Save a snapshot of current data."""
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        filename = f"wifi_snapshot_{timestamp}.json"
        
        # Create snapshot data
        snapshot = {
            "timestamp": timestamp,
            "interface": self.interface,
            "targets": self.targets,
            "wifi_stats": {
                "rssi": self.wifi_stats_widget.rssi,
                "signal_quality": self.wifi_stats_widget.signal_quality,
                "bitrate": self.wifi_stats_widget.bitrate,
                "ssid": self.wifi_stats_widget.ssid
            },
            "ping_results": {
                target: summary.model_dump() if summary else None
                for target, summary in self.ping_table_widget.results.items()
            }
        }
        
        try:
            import json
            with open(filename, "w") as f:
                json.dump(snapshot, f, indent=2)
            self.log_widget.write(f"Snapshot saved to {filename}")
        except Exception as e:
            self.log_widget.write(f"Error saving snapshot: {e}")
    
    def on_exit(self) -> None:
        """Clean up when exiting."""
        self.running = False
        self.log_widget.write("Monitoring stopped.")


if __name__ == "__main__":
    # For testing
    app = LiveViewApp("wlan0", ["8.8.8.8", "1.1.1.1"])
    app.run() 