"""Safe subprocess execution utilities."""

import asyncio
import subprocess
import time
from typing import List, Optional, Tuple, Union
from pathlib import Path

from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn

console = Console()


class ShellError(Exception):
    """Exception raised when a shell command fails."""
    
    def __init__(self, command: List[str], returncode: int, stdout: str, stderr: str):
        self.command = command
        self.returncode = returncode
        self.stdout = stdout
        self.stderr = stderr
        super().__init__(f"Command failed: {' '.join(command)} (exit code: {returncode})")


async def run_command(
    command: List[str],
    timeout: Optional[float] = None,
    capture_output: bool = True,
    check: bool = True,
    cwd: Optional[Union[str, Path]] = None,
    env: Optional[dict] = None,
    show_progress: bool = False,
    progress_text: str = "Running command..."
) -> Tuple[int, str, str]:
    """
    Run a command asynchronously with proper error handling.
    
    Args:
        command: List of command and arguments
        timeout: Timeout in seconds
        capture_output: Whether to capture stdout/stderr
        check: Whether to raise ShellError on non-zero exit code
        cwd: Working directory
        env: Environment variables
        show_progress: Whether to show a progress spinner
        progress_text: Text to show in progress spinner
    
    Returns:
        Tuple of (returncode, stdout, stderr)
    
    Raises:
        ShellError: If command fails and check=True
        asyncio.TimeoutError: If command times out
    """
    if show_progress:
        with Progress(
            SpinnerColumn(),
            TextColumn(progress_text),
            console=console,
            transient=True
        ) as progress:
            task = progress.add_task("", total=None)
            try:
                result = await _run_command_impl(
                    command, timeout, capture_output, cwd, env, check
                )
                progress.update(task, completed=True)
                return result
            except Exception as e:
                progress.update(task, completed=True)
                raise e
    else:
        return await _run_command_impl(command, timeout, capture_output, cwd, env, check)


async def _run_command_impl(
    command: List[str],
    timeout: Optional[float],
    capture_output: bool,
    cwd: Optional[Union[str, Path]],
    env: Optional[dict],
    check: bool = True
) -> Tuple[int, str, str]:
    """Internal implementation of command execution."""
    try:
        # Convert Path to string if needed
        if isinstance(cwd, Path):
            cwd = str(cwd)
        
        # Prepare environment
        process_env = None
        if env:
            process_env = os.environ.copy()
            process_env.update(env)
        
        # Run the command
        process = await asyncio.create_subprocess_exec(
            *command,
            stdout=asyncio.subprocess.PIPE if capture_output else None,
            stderr=asyncio.subprocess.PIPE if capture_output else None,
            cwd=cwd,
            env=process_env
        )
        
        stdout, stderr = await asyncio.wait_for(
            process.communicate(),
            timeout=timeout
        )
        
        returncode = process.returncode
        stdout_str = stdout.decode('utf-8', errors='replace') if stdout else ""
        stderr_str = stderr.decode('utf-8', errors='replace') if stderr else ""
        
        if check and returncode != 0:
            raise ShellError(command, returncode, stdout_str, stderr_str)
        
        return returncode, stdout_str, stderr_str
    
    except asyncio.TimeoutError:
        # Try to terminate the process if it's still running
        try:
            process.terminate()
            await asyncio.wait_for(process.wait(), timeout=5)
        except asyncio.TimeoutError:
            process.kill()
            await process.wait()
        raise asyncio.TimeoutError(f"Command timed out after {timeout} seconds")


def run_command_sync(
    command: List[str],
    timeout: Optional[float] = None,
    capture_output: bool = True,
    check: bool = True,
    cwd: Optional[Union[str, Path]] = None,
    env: Optional[dict] = None,
    input_data: Optional[str] = None
) -> Tuple[int, str, str]:
    """
    Run a command synchronously with proper error handling.
    
    Args:
        command: List of command and arguments
        timeout: Timeout in seconds
        capture_output: Whether to capture stdout/stderr
        check: Whether to raise ShellError on non-zero exit code
        cwd: Working directory
        env: Environment variables
        input_data: Input data to send to stdin
    
    Returns:
        Tuple of (returncode, stdout, stderr)
    
    Raises:
        ShellError: If command fails and check=True
        subprocess.TimeoutExpired: If command times out
    """
    try:
        # Convert Path to string if needed
        if isinstance(cwd, Path):
            cwd = str(cwd)
        
        # Prepare environment
        process_env = None
        if env:
            process_env = os.environ.copy()
            process_env.update(env)
        
        # Prepare input
        stdin_data = None
        if input_data:
            stdin_data = input_data.encode('utf-8')
        
        # Run the command
        result = subprocess.run(
            command,
            capture_output=capture_output,
            text=not capture_output,  # Don't use text mode if not capturing output
            timeout=timeout,
            cwd=cwd,
            env=process_env,
            input=stdin_data,
            check=False  # We'll handle the checking ourselves
        )
        
        if capture_output:
            stdout = result.stdout or ""
            stderr = result.stderr or ""
        else:
            stdout = ""
            stderr = ""
        
        if check and result.returncode != 0:
            raise ShellError(command, result.returncode, stdout, stderr)
        
        return result.returncode, stdout, stderr
    
    except subprocess.TimeoutExpired as e:
        raise subprocess.TimeoutExpired(
            command, timeout, output=e.output, stderr=e.stderr
        )


def which(command: str) -> Optional[str]:
    """Find the full path of a command."""
    import shutil
    return shutil.which(command)


def is_command_available(command: str) -> bool:
    """Check if a command is available in PATH."""
    return which(command) is not None


def get_command_version(command: str) -> Optional[str]:
    """Get the version of a command if available."""
    try:
        # Try common version flags
        for flag in ["--version", "-v", "-V", "version"]:
            try:
                result = subprocess.run(
                    [command, flag],
                    capture_output=True,
                    text=True,
                    timeout=5
                )
                if result.returncode == 0 and result.stdout.strip():
                    # Extract version from output
                    lines = result.stdout.strip().split('\n')
                    for line in lines:
                        if any(char.isdigit() for char in line):
                            return line.strip()
                    return lines[0].strip()
            except (subprocess.TimeoutExpired, FileNotFoundError):
                continue
    except Exception:
        pass
    
    return None


def format_command_error(error: ShellError) -> str:
    """Format a ShellError for display."""
    return (
        f"Command failed: {' '.join(error.command)}\n"
        f"Exit code: {error.returncode}\n"
        f"stdout: {error.stdout}\n"
        f"stderr: {error.stderr}"
    )


# Import os at module level for environment handling
import os 